#include "ks0108.h"

#include <avr/io.h>
#include <util/delay.h>

uint8_t GLCD_posx, GLCD_posy, GLCD_page;

void GLCD_Init()
{
    //Portdirections:
    GLCD_CONT_DIR1 |= (1 << GLCD_RS) | (1 << GLCD_RST) | (1 << GLCD_EN) | (1 << GLCD_CS1) | (1 << GLCD_CS2);
    GLCD_CONT_DIR0 |= (1 << GLCD_RW);

    //Reset:
    GLCD_CONT_OUT1 |=  (1 << GLCD_RST);
    _delay_us(200);
    GLCD_CONT_OUT1 &=~ (1 << GLCD_RST);
    _delay_us(200);
    GLCD_CONT_OUT1 |=  (1 << GLCD_RST);
    _delay_us(200);

    //Enable:
    GLCD_WriteCmd(GLCD_LCDON, GLCD_CHIP1);
    GLCD_WriteCmd(GLCD_LCDON, GLCD_CHIP2);

    GLCD_WriteCmd(GLCD_DSPSTART, GLCD_CHIP1);
    GLCD_WriteCmd(GLCD_DSPSTART, GLCD_CHIP2);

    GLCD_posx = 0;
    GLCD_posy = 0;
    GLCD_page = 0;
}

void GLCD_Clock()
{
    GLCD_CONT_OUT1 |=  (1 << GLCD_EN);
    _delay_us(10);
    GLCD_CONT_OUT1 &=~ (1 << GLCD_EN);
    _delay_us(10);
}

void GLCD_WriteCmd(uint8_t cmd, uint8_t chipsel)
{
    GLCD_DATA_DIR	= GLCD_DDR_OUT;

    GLCD_CONT_OUT1 &=~ ((1 << GLCD_CS1) | (1 << GLCD_CS2));

    if (chipsel == GLCD_CHIP1)
        GLCD_CONT_OUT1 |= (1 << GLCD_CS1);
    else
        GLCD_CONT_OUT1 |= (1 << GLCD_CS2);

    //Command:
    GLCD_CONT_OUT1 &=~ (1 << GLCD_RS);
    GLCD_CONT_OUT0 &=~ (1 << GLCD_RW);

    GLCD_DATA_OUT = cmd;

    GLCD_Clock();

    GLCD_DATA_OUT = GLCD_CLEAR;
}

void GLCD_WriteData(uint8_t data)
{
    GLCD_DATA_DIR	= GLCD_DDR_OUT;

    GLCD_CONT_OUT1 &=~ ((1 << GLCD_CS1) | (1 << GLCD_CS2));

    if (GLCD_posx < 64)
        GLCD_CONT_OUT1 |= (1 << GLCD_CS1);
    else
        GLCD_CONT_OUT1 |= (1 << GLCD_CS2);

    if (GLCD_posx == 64)
        GLCD_WriteCmd(GLCD_SET_X, GLCD_CHIP2);

    //Data:
    GLCD_CONT_OUT1 |=  (1 << GLCD_RS);
    GLCD_CONT_OUT0 &=~ (1 << GLCD_RW);

    GLCD_DATA_OUT	= data;

    GLCD_Clock();

    GLCD_posx++;
}

uint8_t GLCD_ReadData()
{
    uint8_t data;

    GLCD_DATA_OUT = GLCD_DDR_IN;
    GLCD_DATA_DIR = GLCD_DDR_IN;

    GLCD_CONT_OUT1 &=~ ((1 << GLCD_CS1) | (1 << GLCD_CS2));

    if (GLCD_posx < 64)
        GLCD_CONT_OUT1 |= (1 << GLCD_CS1);
    else
        GLCD_CONT_OUT1 |= (1 << GLCD_CS2);

    if (GLCD_posx == 64)
        GLCD_WriteCmd(GLCD_SET_X, GLCD_CHIP2);

    //Read Data:
    GLCD_CONT_OUT1 |= (1 << GLCD_RS);
    GLCD_CONT_OUT0 |= (1 << GLCD_RW);

    GLCD_Clock();
    GLCD_Clock();

    data = GLCD_DATA_IN;

    GLCD_DATA_DIR = GLCD_DDR_OUT;
    GLCD_DATA_OUT = GLCD_CLEAR;

    GLCD_GotoXY(GLCD_posx, GLCD_posy);

    return data;
}

void GLCD_GotoXY(uint8_t x, uint8_t y)
{
    uint8_t	chip = GLCD_CHIP1;

    GLCD_posx = x;
    GLCD_posy = y;
    GLCD_page = (y >> 3);

    if (x > 63)
    {
        x -= 64;
        chip = GLCD_CHIP2;
    }

    GLCD_WriteCmd (0x40 | x, chip);
    GLCD_WriteCmd (0xB8 | (y >> 3), GLCD_CHIP1);
    GLCD_WriteCmd (0xB8 | (y >> 3), GLCD_CHIP2);
}


void GLCD_Fill(uint8_t Mode)
{
    uint8_t x, y;
    for (y = 0; y < 8; y++)
    {
        GLCD_GotoXY (0, (y << 3));

        for (x = 0; x < 128; x++)
        {
            GLCD_WriteData(Mode);
        }
    }
}

void GLCD_SetPixel(uint8_t	x, uint8_t y)
{
    uint8_t	cmd;

    GLCD_GotoXY (x, y);
    cmd  = GLCD_ReadData ();
    cmd |= (1 << (y % 8));

    GLCD_WriteData (cmd);
}

uint8_t sgn(uint8_t x)
{
    return (x > 0) ? 1 : (x < 0) ? -1 : 0;
}


void GLCD_DrawLine(uint8_t xstart,uint8_t ystart,uint8_t xend,uint8_t yend)
{
    int16_t x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;

    // Entfernung in beiden Dimensionen berechnen
    dx = xend - xstart;
    dy = yend - ystart;

    // Vorzeichen des Inkrements bestimmen
    incx = sgn(dx);
    incy = sgn(dy);
    if (dx<0) dx = -dx;
    if (dy<0) dy = -dy;

    // feststellen, welche Entfernung größer ist
    if (dx>dy)
    {
        // x ist schnelle Richtung
        pdx=incx;
        pdy=0;    // pd. ist Parallelschritt
        ddx=incx;
        ddy=incy; // dd. ist Diagonalschritt
        es =dy;
        el =dx;   // Fehlerschritte schnell, langsam
    }
    else
    {
        // y ist schnelle Richtung
        pdx=0;
        pdy=incy; // pd. ist Parallelschritt
        ddx=incx;
        ddy=incy; // dd. ist Diagonalschritt
        es =dx;
        el =dy;   // Fehlerschritte schnell, langsam
    }

    // Initialisierungen vor Schleifenbeginn
    x = xstart;
    y = ystart;
    err = el/2;
    GLCD_SetPixel(x,y);

    // Pixel berechnen
    for (t=0; t<el; ++t) // t zaehlt die Pixel, el ist auch Anzahl
    {
        // Aktualisierung Fehlerterm
        err -= es;
        if (err<0)
        {
            // Fehlerterm wieder positiv (>=0) machen
            err += el;
            // Schritt in langsame Richtung, Diagonalschritt
            x += ddx;
            y += ddy;
        }
        else
        {
            // Schritt in schnelle Richtung, Parallelschritt
            x += pdx;
            y += pdy;
        }
        GLCD_SetPixel(x,y);
    }

}

void GLCD_DrawBox(uint8_t x, uint8_t y, uint8_t width, uint8_t height)
{
    uint8_t dx, dy;

    for (dx = x; dx < x+width; dx++)
    {
        GLCD_SetPixel(dx,y);
        GLCD_SetPixel(dx,y+height);
    }

    for (dy = y; dy < y+height; dy++)
    {
        GLCD_SetPixel(x,dy);
        GLCD_SetPixel(x+width,dy);
    }
}

