/**************************************************************************

    jyetech digital storage oscilloscope (http://www.http://jyetech.com/)
    alternative firmware

    Copyright (C) 2009  Franz Neumann
    mail: netinside2000@gmx.de
    web: http://www.netinside2000.de

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

***************************************************************************/

#define F_CPU 20000000UL

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "ks0108.h"

#define MAINJOB    0
#define KEYCHECK   1
#define TRIGGER    2
#define SAMPLING   3
#define SAMPLEDRAW 4
#define PREPSCREEN 5
#define BLINKTEST  6

#define MENU_NONE  0
#define MENU_VPOS  1
#define MENU_TIME  2
#define MENU_TRIG  3

#define KEY_NONE   0
#define KEY_OK     1
#define KEY_PLUS   2
#define KEY_MINUS  3
#define KEY_VPOS   4
#define KEY_HPOS   5
#define KEY_TIME   6
#define KEY_MODE   7
#define KEY_SLOPE  8
#define KEY_LEVEL  9

uint8_t GetKey()
{
    //Row as Input:
    DDRD &=~ ( (1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3) );
    //Pullup Inputs:
    PORTD |= ( (1 << PD0) | (1 << PD1) | (1 << PD2) | (1 << PD3) );

    //Columns Output:
    DDRF |= ( (1 << PF5) | (1 << PF6) | (1 << PF7) );
    //Colums high:
    PORTF |= ( (1 << PF5) | (1 << PF6) | (1 << PF7) );

    //Col0 low
    PORTF &=~ (1 << PF5);
    _delay_us(10);
    if (!(PIND & (1 << PD0))) return KEY_OK; //SW6
    PORTF |= (1 << PF5);

    //Col1 low
    PORTF &=~ (1 << PF6);
    _delay_us(10);
    if (!(PIND & (1 << PD0))) return KEY_PLUS; //SW5
    if (!(PIND & (1 << PD1))) return KEY_TIME; //SW8
    if (!(PIND & (1 << PD2))) return KEY_HPOS; //SW10
    if (!(PIND & (1 << PD3))) return KEY_VPOS; //SW12
    PORTF |= (1 << PF6);

    //Col2 low
    PORTF &=~ (1 << PF7);
    _delay_us(10);
    if (!(PIND & (1 << PD0))) return KEY_MINUS; //SW4
    if (!(PIND & (1 << PD1))) return KEY_LEVEL; //SW7
    if (!(PIND & (1 << PD2))) return KEY_SLOPE; //SW9
    if (!(PIND & (1 << PD3))) return KEY_MODE; //SW11
    PORTF |= (1 << PF7);

    return KEY_NONE;
}

int main(void)
{
#define MAX 128

    uint8_t i = 0;
    uint8_t val[MAX], val0, val1;
    uint16_t s_delay = 0;
    uint8_t triglevel = 128;
    uint8_t machine_next = MAINJOB, keypressed = KEY_NONE, menumode = MENU_NONE;

    GLCD_Init();
    GLCD_Fill (GLCD_CLEAR);


    //ADC Clock Pin:
    DDRB |= (1 << PB7);
    //ADC Data Input:
    PORTA = 0x00;
    DDRA  = 0x00; //No Pullup

    //setting PWM-Ports as output
    DDRE |= (1 << PE3)|(1 << PE4);
    //Fast PWM Mode, ICR3 = Top, OCR = Duty Cycle:
    TCCR3A = (1<<COM3A1) | (1<<COM3B1) | (1<<WGM31);
    TCCR3B = (1<<WGM33) | (1<<WGM32) | (0<<CS32) | (0<<CS31) | (1<<CS30); // Clkdiv 1
    //Frequency:
    ICR3 = 255; // 20MHz/ICR3 = 78kHz PWM Frequency
    //Duty Cycle:
    OCR3A = ICR3/2.7; //V- = -9V, V+ = 9V
    OCR3B = ICR3/2.7; //Vpos = middle

    while (1)
    {
        switch (machine_next)
        {
        case MAINJOB:
            keypressed = GetKey();

            switch (keypressed)
            {
            case KEY_NONE:
                break;

            case KEY_VPOS:
                menumode = MENU_VPOS;
                break;
            case KEY_TIME:
                menumode = MENU_TIME;
                break;
            case KEY_LEVEL:
                menumode = MENU_TRIG;
                break;

            case KEY_PLUS:
                switch (menumode)
                {
                case MENU_VPOS:
                    if (OCR3B < 255)
                        OCR3B++;
                    break;
                case MENU_TIME:
                    if (s_delay < 255)
                        s_delay++;
                    break;
                case MENU_TRIG:
                    if (triglevel < 255)
                        triglevel++;
                    break;
                }
                break;

            case KEY_MINUS:
                switch (menumode)
                {
                case MENU_VPOS:
                    if (OCR3B > 0)
                        OCR3B--;
                    break;
                case MENU_TIME:
                    if (s_delay > 0)
                        s_delay--;
                    break;
                case MENU_TRIG:
                    if (triglevel > 0)
                        triglevel--;
                    break;
                }
                break;
            }

            machine_next = TRIGGER;
            break;

        case TRIGGER:
            PORTB |= (1 << PB7);
            val0 = PINA;
            PORTB &=~ (1 << PB7);

            _delay_us(5*s_delay);

            PORTB |= (1 << PB7);
            val1 = PINA;
            PORTB &=~ (1 << PB7);

            if ( (val0 > triglevel) && (val1 > val0) )
            {
                machine_next = SAMPLING;
            }
            else
            {
                machine_next = PREPSCREEN;
            }
            break;

        case SAMPLING:
            for (i = 0; i < MAX; i++)
            {
                PORTB |= (1 << PB7);
                val[i] = PINA;
                PORTB &=~ (1 << PB7);

                _delay_us(s_delay);
            }

            machine_next = PREPSCREEN;
            break;

        case PREPSCREEN:
            GLCD_Fill (GLCD_CLEAR);
            GLCD_DrawBox(0,0,127,63);
            GLCD_DrawBox(0,triglevel/4-1,2,3);

            machine_next = SAMPLEDRAW;
            break;

        case SAMPLEDRAW:
            for (i = 0; i < 128; i++)
            {

                //if(i < 127)
                //GLCD_DrawLine(i,val[i]/4,i+1,val[i+1]/4);
                //else
                GLCD_SetPixel(i, val[i]/4-1);
            }

            machine_next = MAINJOB;
            break;

        }
    }

    return 0;
}

